use std::sync::Mutex;

use rocket::{
    http::Status,
    serde::{json::Json, Serialize},
    Build, Rocket, State,
};

use crate::{blackjack::{blackjack_game::BlackjackGame, gamestate::GameState, play_moves::PlayMoves, player::Player}, cards::card::Card};

#[get("/state")]
fn get_state(state: &State<MyState>) -> Json<GamestateJson> {
    Json(gamestate_as_json(&state.game.lock().unwrap()))
}

#[post("/state", data = "<request>", format = "application/json")]
fn post_move(
    state: &State<MyState>,
    request: Json<PlayMoves>,
) -> Result<Json<GamestateJson>, Status> {
    let action = request.0;

    if state.game.lock().unwrap().play(action) {
        Ok(Json(gamestate_as_json(&state.game.lock().unwrap())))
    } else {
        Err(Status::BadRequest)
    }
}

struct MyState {
    game: Mutex<BlackjackGame>,
}

#[derive(Serialize)]
struct GamestateJson {
    state: GameState,
    dealer_upcard: Option<Card>,
    players: Vec<Player>,
}

fn gamestate_as_json(game: &BlackjackGame) -> GamestateJson {
    match game.get_state() {
        GameState::Starting => GamestateJson {
            state: *game.get_state(),
            dealer_upcard: None,
            players: Vec::new(),
        },
        GameState::Over | GameState::PlayerTurn { .. } => GamestateJson {
            state: *game.get_state(),
            dealer_upcard: game.get_dealer_upcard().copied(),
            players: game.get_players().clone(),
        },
    }
}

pub fn build() -> Rocket<Build> {
    let state = MyState {
        game: Mutex::new(BlackjackGame::new()),
    };

    rocket::build()
        .manage(state)
        .mount("/", routes![get_state, post_move])
}
